/**
 * @fileOverview 本地调试服务器
 * @author langford@126.com
 */

const webpack = require('webpack')
const _ = require('lodash')
const path = require('path')
const chalk = require('chalk')
const serve = require('webpack-serve')
const convert = require('koa-connect')
const history = require('connect-history-api-fallback')
const proxy = require('http-proxy-middleware')
const devConfig = require('../webpack/webpack.dev.config')
const utils = require('../utils')

const abcConfig = utils.abcConfig
const port = abcConfig.port
const proxyConfig = utils.getProxyConfig()

const compile = webpack(devConfig)

serve(
  {},
  {
    compiler: compile,
    host: abcConfig.host,
    open: true,
    port: port,
    content: [abcConfig.staticPath || process.cwd()],
    add: (app, middleware, options) => {
      _.forEach(proxyConfig, (config) => {
        const { context, ...options } = config

        app.use(convert(proxy(context, options)))
      })

      app.use(
        convert(
          history({ whiteList: _.merge([], abcConfig.history.whiteList) })
        )
      )
    },
    devMiddleware: {
      stats: {
        colors: true,
        moduleTrace: false,
        modules: false
      },
      // watchOptions: {
      //   aggregateTimeout: 100,
      //   poll: true
      // },
      quiet: true
    },
    hotClient: true
  }
).then((result) => {})
