/**
 * @fileOverview webpack开发配置
 * @author langford@126.com
 */

"use strict"

const webpack = require("webpack")
const _ = require("lodash")
const HtmlWebpackPlugin = require("html-webpack-plugin")
const config = require("./webpack.base.config")({
  dev: true,
  env: "development"
})
const utils = require("../utils")
const abcConfig = utils.abcConfig
const devConfig = abcConfig.env.development
const FriendlyErrorsWebpackPlugin = require("friendly-errors-webpack-plugin")
const ErrorOverlayPlugin = require("error-overlay-webpack-plugin")

// 合并 development rules
const rules = [
  [
    {
      test: /\.css$/,
      use: [
        "style",
        "css",
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        }
      ]
    },
    {
      test: /\.less$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            localIdentName: abcConfig.cssModule.localIdentName,
            importLoaders: 2,
            sourcemap: true
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        {
          loader: "less",
          options: {
            javascriptEnabled: true
          }
        }
      ]
    },
    {
      test: /\.s(c|a)ss$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            localIdentName: abcConfig.cssModule.localIdentName
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        "sass"
      ]
    },
    {
      test: /\.styl(us)?$/,
      use: [
        "vue-style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            minimize: true,
            localIdentName: abcConfig.cssModule.localIdentName
          }
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions()
        },
        "stylus"
      ]
    }
  ],
  config.module.rules,
  devConfig.rules
]
config.module.rules = utils.merge(rules, "rule")

// 合并 development plugins
const plugins = [
  [
    new webpack.DefinePlugin({
      __DEV__: true,
      NODE_ENV: JSON.stringify("development")
    }),
    new FriendlyErrorsWebpackPlugin()
  ]
    .concat(abcConfig.disableErrorOverlay ? [] : [new ErrorOverlayPlugin()])
    .concat(
      abcConfig.enableHtmlWebpackPlugin
        ? [
            new HtmlWebpackPlugin({
              template: devConfig.htmlTemplateUrl || abcConfig.htmlTemplateUrl,
              inject: "body",
              ...abcConfig.htmlWebpackPluginOptions,
              ...devConfig.htmlWebpackPluginOptions
            })
          ]
        : []
    ),
  config.plugins,
  devConfig.plugins
]

config.plugins = utils.merge(plugins, "plugin")

config.mode = "development"
module.exports = config
